#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <unistd.h>

#include "../interface_2001.h"

#define BAUDRATE B57600

/* since the actual power output by the lights is the voltage^2, and the 
   voltage is a sine wave, the delay we tell the box needs to be scaled to
   produce a linear brightness in relation to linear value increase.
   
   The power produced by a specific delay is P=x/2-(sin(x)*cos(x))/2 where x is
   angle in radians after zero crossing... the max power occurs with 0 delay
   
   I produced this table by solving the above equation for P=PI/512 P=2*PI/512,
   P=3*PI/512, etc...
*/
int scale_table[256]={
 1, 4, 5, 6, 7, 7, 8, 8, 8, 9, 9, 9, 10, 10, 10, 10,
 11, 11, 11, 11, 12, 12, 12, 12, 12, 13, 13, 13, 13, 13, 13, 14,
 14, 14, 14, 14, 14, 14, 15, 15, 15, 15, 15, 15, 15, 16, 16, 16,
 16, 16, 16, 16, 17, 17, 17, 17, 17, 17, 17, 17, 18, 18, 18, 18,
 18, 18, 18, 18, 18, 19, 19, 19, 19, 19, 19, 19, 19, 19, 20, 20,
 20, 20, 20, 20, 20, 20, 20, 21, 21, 21, 21, 21, 21, 21, 21, 21,
 21, 22, 22, 22, 22, 22, 22, 22, 22, 22, 22, 23, 23, 23, 23, 23,
 23, 23, 23, 23, 23, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 26, 26, 26, 26, 26, 26,
 26, 26, 26, 26, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 28, 28,
 28, 28, 28, 28, 28, 28, 28, 28, 29, 29, 29, 29, 29, 29, 29, 29,
 29, 30, 30, 30, 30, 30, 30, 30, 30, 30, 31, 31, 31, 31, 31, 31,
 31, 31, 31, 32, 32, 32, 32, 32, 32, 32, 32, 33, 33, 33, 33, 33,
 33, 33, 33, 34, 34, 34, 34, 34, 34, 34, 35, 35, 35, 35, 35, 35,
 36, 36, 36, 36, 36, 36, 37, 37, 37, 37, 37, 38, 38, 38, 38, 39,
 39, 39, 39, 40, 40, 40, 41, 41, 41, 42, 42, 43, 43, 44, 45, 49,
  };

static int box_locations[60][2];	// contains the box number, and plug number that
									// is located at each position on the hall

static int box_fd=-1;
static int light_data[NUM_SEGMENTS*6+1];
static int mapped_light_data[NUM_SEGMENTS*6+1]; // the light data after it has
											  // been mapped to the actual hardware
static char modem_device[]="/dev/ttyS0";

int box_write_reg(int box, int regpair, int a, int b);
int sendbyte(int byte, int address);
int set_light_intensity(int box, int regpair, int a, int b);

int init_lights()
{
	struct termios tio;
	FILE *config;
	int i;

	box_fd=open(modem_device, O_RDWR | O_NOCTTY);
	if (box_fd < 0) {
		perror(modem_device);
		return -1;
	}
	
	memset(&tio,0,sizeof(struct termios));
	tio.c_cflag = BAUDRATE | CS8 | CLOCAL | PARENB | CREAD | PARODD;
	tio.c_iflag = IGNPAR;
	
	if (tcsetattr(box_fd, TCSANOW, &tio)<0) {
		perror("tcsetattr");
		close(box_fd);
		box_fd=-1;
		return -1;
	}
	
	// try to open configuration file...
	
	config=fopen("hall_config","r");
	if (config==NULL) {
		// we need to give default values
		for (i=0;i<60;i++) {
			box_locations[i][0]=i/6; 
			box_locations[i][1]=i%6;
		}
	} else {
		for (i=0;i<60;i++) {
			fscanf(config,"%i",&box_locations[i][0]);
			fscanf(config,"%i",&box_locations[i][1]);
		}
		fclose(config);
	}	
	
	return 0;
}

int deinit_lights()
{
	if (box_fd>0) {
		close(box_fd);
		return 0;
	}
	
	return -1;
}

/*void light_state(int light, int state)
{
	int box,ns;
	
	box=light+1;
	ns=state*255;

	set_light_intensity(box,0,ns,ns);
	set_light_intensity(box,1,ns,ns);
	set_light_intensity(box,2,ns,ns);

	light_data[light*6]=ns; light_data[light*6+1]=ns;
	light_data[light*6+2]=ns; light_data[light*6+3]=ns;
	light_data[light*6+4]=ns; light_data[light*6+5]=ns;	
}*/

// takes a ideal mapping, and puts the values where they need to be in
// mapped_light_data
int map_frame(int *data)	
{
	int i;
	int x;
	for (i=0;i<60;i++) {
//		printf("Mapping box:%i plug:%i to position:%i\n",box_locations[i][0],
//			box_locations[i][1],i);
		x=(box_locations[i][0]-1)*6+box_locations[i][1]-1;
		if (x>59) {
			fprintf(stderr,"BAD MAPPING CODE!!\n\n\n");
			x=59;
		}
		mapped_light_data[(box_locations[i][0]-1)*6+(box_locations[i][1]-1)]=data[i];
	}	
	return 0;
}

int renderFrame(int *data)
{
	// look through the data and write out any regpairs that differ from the last frame
	int i;
//	unsigned char buf[60/2*3];
	
	map_frame(data);
	for (i=0;i<(NUM_SEGMENTS*6);i+=2) {
		if (set_light_intensity(i/6+1,(i%6)/2,mapped_light_data[i],
				mapped_light_data[i+1])<0) return -1;
		light_data[i]=mapped_light_data[i]; light_data[i+1]=mapped_light_data[i+1];
	}
	
//	write(box_fd,&buf,60/2*3);
	return 0;
}

int set_light_intensity(int box, int regpair, int a, int b)
{
	// wrapper around box_write_reg... performs linearization on the intensity
	// parameters to make 255 be the brightest and 0 be the darkest
	
	return box_write_reg(box,regpair,scale_table[255-a],scale_table[255-b]);
//	data[0]=box<<1 | regpair | 128;
//	data[1]=scale_table[255-a];
//	data[2]=scale_table[255-b];
	return 0;
}	

int box_write_reg(int box, int regpair, int a, int b)
{
	int ret=0;
    ret+=sendbyte((box<<2) | regpair, 1); 
    ret+=sendbyte(a,0);
    ret+=sendbyte(b,0);
    return ret;
}

int sendbyte(int byte, int address)
{
    unsigned char c;
 
    if (address==1) c=byte|128; else c=byte&127;

    if (write(box_fd,&c,1)<0) {
		perror("write");
		return -1;
	}

    return 0;
}   
